/*
     Copyright 2007 Joseph M. Ferner

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
using System.IO;
using log4net;
using OpenServers.Net.Common.FileSystem;
using OpenServers.Net.FtpServer.State;

namespace OpenServers.Net.FtpServer.Command {
    /// <summary>
    ///     <para>
    ///         <b>CHANGE WORKING DIRECTORY (CWD)</b><br/>
    ///         This command allows the user to work with a different
    ///         directory or dataset for file storage or retrieval without
    ///         altering his login or accounting information.  Transfer
    ///         parameters are similarly unchanged.  The argument is a
    ///         pathname specifying a directory or other system dependent
    ///         file group designator.
    ///     </para>
    ///     <para>
    ///         <b>Format</b><br/>
    ///         <code>CWD &lt;SP&gt; &lt;pathname&gt; &lt;CRLF&gt;</code>
    ///     </para>
    ///     <para>
    ///         <b>Success:</b>
    ///         <list type="bullet">
    ///             <item><term>250</term><description>Requested file action okay, completed.</description></item>
    ///         </list>
    ///     </para>
    ///     <para>
    ///         <b>Errors:</b>
    ///         <list type="bullet">
    ///             <item><term>500</term><description>Syntax error, command unrecognized.</description></item>
    ///             <item><term>501</term><description>Syntax error in parameters or arguments.</description></item>
    ///             <item><term>502</term><description>Command not implemented.</description></item>
    ///             <item><term>421</term><description>Service not available, closing control connection.</description></item>
    ///             <item><term>530</term><description>Not logged in.</description></item>
    ///             <item><term>550</term><description>Requested action not taken. File unavailable (e.g., file not found, no access).</description></item>
    ///         </list>
    ///     </para>
    /// </summary>
    public class ChangeWorkingDirectoryCommand : BaseCommand {
        private static readonly ILog log = LogManager.GetLogger(typeof(ChangeWorkingDirectoryCommand));

        /// <summary>
        /// changes the current working directory.
        /// </summary>
        public override void PerformCommand(IFtpServer server, FtpState state, string arguments) {
            if (string.IsNullOrEmpty(arguments)) {
                state.Listener.Send(state, server.Messages.Get501());
            } else {
                string newPath = GetFilePath(state, arguments);
                newPath = Path.GetFullPath(newPath);
                newPath = newPath.Substring(Path.GetPathRoot(newPath).Length - 1);
                newPath = newPath.Replace('\\', '/');
                IDirectory fso = server.FileSystem.GetDirectory(newPath);
                if (server.SecurityProvider.CanRead(state.User, fso)) {
                    state.CurrentDirectory = newPath;
                    log.DebugFormat("new path is [{0}]", state.CurrentDirectory);
                    state.Listener.Send(state, server.Messages.Get250());
                } else {
                    state.Listener.Send(state, server.Messages.Get550(Reason550.AccessDenied));
                }
            }
        }
    }
}
