/*
     Copyright 2007 Joseph M. Ferner

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

using OpenServers.Net.Common.FileSystem;
using OpenServers.Net.FtpServer.State;

namespace OpenServers.Net.FtpServer.Command {
    /// <summary>
    ///     <para>
    ///         <b>DELETE (DELE)</b><br/>
    ///         This command causes the file specified in the pathname to be
    ///         deleted at the server site.  If an extra level of protection
    ///         is desired (such as the query, "Do you really wish to
    ///         delete?"), it should be provided by the user-FTP process.
    ///     </para>
    ///     <para>
    ///         <b>Format</b><br/>
    ///         <code>DELE &lt;SP&gt; &lt;pathname&gt; &lt;CRLF&gt;</code>
    ///     </para>
    ///     <para>
    ///         <b>Success:</b>
    ///         <list type="bullet">
    ///             <item><term>250</term><description>Requested file action okay, completed.</description></item>
    ///         </list>
    ///     </para>
    ///     <para>
    ///         <b>Errors:</b>
    ///         <list type="bullet">
    ///             <item><term>421</term><description>Service not available, closing control connection.</description></item>
    ///             <item><term>450</term><description>Requested file action not taken. File unavailable (e.g., file busy).</description></item>
    ///             <item><term>500</term><description>Syntax error, command unrecognized.</description></item>
    ///             <item><term>501</term><description>Syntax error in parameters or arguments.</description></item>
    ///             <item><term>502</term><description>Command not implemented.</description></item>
    ///             <item><term>530</term><description>Not logged in.</description></item>
    ///             <item><term>550</term><description>Requested action not taken. File unavailable (e.g., file not found, no access).</description></item>
    ///         </list>
    ///     </para>
    /// </summary>
    public class DeleteCommand : BaseCommand {
        /// <summary>
        /// deletes a file/directory.
        /// </summary>
        public override void PerformCommand(IFtpServer server, FtpState state, string arguments) {
            if (string.IsNullOrEmpty(arguments)) {
                state.Listener.Send(state, server.Messages.Get501());
            } else {
                string filePath = GetFilePath(state, arguments);
                IFile file = server.FileSystem.GetFile(filePath);
                if (file == null || !file.Exists) {
                    state.Listener.Send(state, server.Messages.Get550(Reason550.FileNotFound));
                } else {
                    if (server.SecurityProvider.CanWrite(state.User, file.Parent)) {
                        file.Delete();
                        state.Listener.Send(state, server.Messages.Get250());
                    } else {
                        state.Listener.Send(state, server.Messages.Get550(Reason550.AccessDenied));
                    }
                }
            }
        }
    }
}
