/*
     Copyright 2007 Joseph M. Ferner

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

using NMock2;
using NUnit.Framework;
using OpenServers.Net.Common.FileSystem;

namespace OpenServers.Net.FtpServer.Command {
    [TestFixture]
    public class RenameToCommandTest : AbstractCommandTest<RenameToCommand> {
        [Test]
        public void TestInvoke() {
            IFile fileSrc = Mocks.NewMock<IFile>();
            IFile fileDest = Mocks.NewMock<IFile>();

            State.LastRenameFrom = fileSrc;

            Expect.Once.On(MockFileSystem).Method("GetFile").With("/a.txt").Will(Return.Value(fileDest));
            Expect.Once.On(fileDest).GetProperty("Exists").Will(Return.Value(false));
            Expect.Once.On(MockSecurityProvider).Method("CanWrite").With(MockUser, fileDest).Will(Return.Value(true));
            Expect.Once.On(fileSrc).Method("MoveTo").With(fileDest);
            Expect.Once.On(MockMessages).Method("Get250");

            PrivateObject.WrappedObject.Invoke(MockFtpServer, State, "a.txt");
        }

        [Test]
        public void TestInvokeBadRenameFrom() {
            IFileSystemObject fileSrc = Mocks.NewMock<IFileSystemObject>();

            State.LastRenameFrom = fileSrc;

            Expect.Once.On(MockMessages).Method("Get553");

            PrivateObject.WrappedObject.Invoke(MockFtpServer, State, "a.txt");
        }

        [Test]
        public void TestInvokeSourceIsDirectory() {
            IDirectory dirSrc = Mocks.NewMock<IDirectory>();
            IDirectory dirDest = Mocks.NewMock<IDirectory>();

            State.LastRenameFrom = dirSrc;

            Expect.Once.On(MockFileSystem).Method("GetDirectory").With("/test").Will(Return.Value(dirDest));
            Expect.Once.On(dirDest).GetProperty("Exists").Will(Return.Value(false));
            Expect.Once.On(MockSecurityProvider).Method("CanWrite").With(MockUser, dirDest).Will(Return.Value(true));
            Expect.Once.On(dirSrc).Method("MoveTo").With(dirDest);
            Expect.Once.On(MockMessages).Method("Get250");

            PrivateObject.WrappedObject.Invoke(MockFtpServer, State, "test");
        }

        [Test]
        public void TestInvokeMissingRenameFrom() {
            State.LastRenameFrom = null;

            Expect.Once.On(MockMessages).Method("Get503");

            PrivateObject.WrappedObject.Invoke(MockFtpServer, State, "a.txt");
        }

        [Test]
        public void TestInvokeFileExists() {
            IFile fileSrc = Mocks.NewMock<IFile>();
            IFile fileDest = Mocks.NewMock<IFile>();

            State.LastRenameFrom = fileSrc;

            Expect.Once.On(MockFileSystem).Method("GetFile").With("/a.txt").Will(Return.Value(fileDest));
            Expect.Once.On(fileDest).GetProperty("Exists").Will(Return.Value(true));
            Expect.Once.On(MockMessages).Method("Get550").With(Reason550.FileAlreadyExists);

            PrivateObject.WrappedObject.Invoke(MockFtpServer, State, "a.txt");
        }

        [Test]
        public void TestInvokeCanWrite() {
            IFile fileSrc = Mocks.NewMock<IFile>();
            IFile fileDest = Mocks.NewMock<IFile>();

            State.LastRenameFrom = fileSrc;

            Expect.Once.On(MockFileSystem).Method("GetFile").With("/a.txt").Will(Return.Value(fileDest));
            Expect.Once.On(fileDest).GetProperty("Exists").Will(Return.Value(false));
            Expect.Once.On(MockSecurityProvider).Method("CanWrite").With(MockUser, fileDest).Will(Return.Value(false));
            Expect.Once.On(MockMessages).Method("Get550").With(Reason550.AccessDenied);

            PrivateObject.WrappedObject.Invoke(MockFtpServer, State, "a.txt");
        }

        [Test]
        public void TestInvokeBadParameter() {
            Expect.Once.On(MockMessages).Method("Get501");

            PrivateObject.WrappedObject.Invoke(MockFtpServer, State, "");
        }

        [Test]
        public void TestInvokeNotLoggedIn() {
            State.User = null;
            Expect.Once.On(MockMessages).Method("Get530");
            PrivateObject.WrappedObject.Invoke(MockFtpServer, State, "a.txt");
        }
    }
}
